@@ -2,6 +2,7 @@
 #include "KasumiWord.hxx"
 #include "KasumiString.hxx"
 #include "KasumiException.hxx"
+#include <stdio.h>
 #include <iostream>
 #include <fstream>
 #include <sstream>
@@ -12,7 +13,9 @@ using namespace std;
 
 #define OptionOutput( Word, OptionName ) (string(OptionName) + " = " + (Word->getOption(OptionName) ? "y" : "n"))
 
-KasumiDic::KasumiDic(const string aDicFileName) throw(KasumiDicExaminationException){
+KasumiDic::KasumiDic(const string aDicFileName)
+  throw(KasumiDicExaminationException){
+
   DicFileName = aDicFileName;
 
   try{
@@ -22,22 +25,27 @@ KasumiDic::KasumiDic(const string aDicFileName) throw(KasumiDicExaminationExcept
   }
 }
 
-void KasumiDic::load() throw(KasumiDicExaminationException){
+void KasumiDic::load()
+  throw(KasumiDicExaminationException){
+  
   int line = 0;
+  int freq;
   string DicContents = string();
   KasumiString Buffer;
 
-  ifstream DicFile(DicFileName.c_str());
-
-  if(!DicFile.is_open()){
-    DicFile.close();    
-    ofstream NewDicFile(DicFileName.c_str());
-    NewDicFile.close();
-    DicFile.open(DicFileName.c_str());
+  string command = "anthy-dic-tool --dump > " + DicFileName;
+  if(system(command.c_str()) != 0){
+    throw KasumiDicExaminationException(
+            "Cannot dump Anthy dictoinary to " + DicFileName,
+            0);
   }
+
+  ifstream DicFile(DicFileName.c_str());
   
   if(!DicFile.is_open()){
-    throw KasumiDicExaminationException("Cannot open Anthy dicitionary file.", 0);
+    throw KasumiDicExaminationException(
+            "Cannot open Anthy dicitionary file.",
+            0);
   }
 
   // analyze Anthy Dictionary reading each line
@@ -52,7 +60,14 @@ void KasumiDic::load() throw(KasumiDicExaminationException){
       KasumiWord *newWord = new KasumiWord();
 
       newWord->setSound(Buffer.getSound());
-      newWord->setFrequency(Buffer.getFrequency());
+      freq = Buffer.getFrequency();
+      if(freq < FREQ_LBOUND){
+        newWord->setFrequency(FREQ_LBOUND);
+      }else if(freq > FREQ_UBOUND){
+        newWord->setFrequency(FREQ_UBOUND);
+      }else{
+        newWord->setFrequency(freq);
+      }
       newWord->setSpelling(Buffer.getSpelling());
 
       while(getline(DicFile, Buffer, '\n')){
@@ -77,20 +92,25 @@ void KasumiDic::load() throw(KasumiDicExaminationException){
             }
           }
         }else{
-          throw KasumiDicExaminationException("Invalid entry in Anthy private dictionary file.", line);
+          throw KasumiDicExaminationException(
+                  "Invalid entry in Anthy private dictionary file.",
+                  line);
         }
       }
 
       appendWord(newWord);
     }else{
       // not classfied line; Anthy Dicitionary is invalid!
-          throw KasumiDicExaminationException("Invalid entry in Anthy private dictionary file.", line);
+          throw KasumiDicExaminationException(
+                  "Invalid entry in Anthy private dictionary file.",
+                  line);
     }
   }
+
 }
 
 int KasumiDic::appendWord(KasumiWord *word){
-  int i;
+  size_t i;
   
   WordList.push_back(word);
   
@@ -101,40 +121,49 @@ int KasumiDic::appendWord(KasumiWord *word){
   return getUpperBoundOfWordID();
 }
 
-void KasumiDic::removeWord(int id) throw(KasumiOutOfBoundException){
-  int i;
+void KasumiDic::removeWord(size_t id)
+  throw(KasumiOutOfBoundException){
+
+  size_t i;
   
   if(id >= WordList.size() || id < 0){
     throw new KasumiOutOfBoundException("Out of Bound!");
   }
 
   KasumiWord *word = WordList[id];
-  WordList[id] = NULL;
+  //  WordList[id] = NULL;
+  word->setFrequency(0);
 
   for(i=0;i<EventListeners.size();i++){
     EventListeners[i]->removedWord(getUpperBoundOfWordID());
   }
   
-  free(word);
+  //  free(word);
 }
 
-void KasumiDic::store() throw(KasumiDicStoreException){
-  int i;
+void KasumiDic::store()
+  throw(KasumiDicStoreException){
+
+  size_t i;
   ofstream DicFile(DicFileName.c_str());
   if(!DicFile.is_open()){
     cout << "Cannot overwrite data to" << DicFileName << "." << endl;
     return;
   }
   string ret = string();
-  
+
   for(i=0; i<WordList.size(); i++){
     ostringstream ostr;
 
+    if(WordList[i] == NULL)
+      continue;
+
     ret += WordList[i]->getSound() + " ";
     ostr << WordList[i]->getFrequency();
     ret += ostr.str() + " ";
     ret += WordList[i]->getSpelling() + "\n";
-    ret += string(EUCJP_HINNSHI) + " = " + WordList[i]->getStringOfWordClass() + "\n";
+    ret += string(EUCJP_HINNSHI) + " = " +
+      WordList[i]->getStringOfWordClass() + "\n";
 
     if(WordList[i]->getWordClass() == NOUN){
       ret += OptionOutput(WordList[i], EUCJP_NASETSUZOKU) + "\n";
@@ -155,12 +184,15 @@ void KasumiDic::store() throw(KasumiDicStoreException){
 
       // nothing to do
     }else{
-      throw KasumiDicStoreException("internal error while saving Anthy Dictionary.");
+      throw KasumiDicStoreException(
+              "internal error while saving Anthy Dictionary.");
     }
     
     ret += "\n";
   }
 
+  cout << ret;
+
   DicFile << ret;
   DicFile.close();
 
@@ -177,7 +209,9 @@ void KasumiDic::registerEventListener(KasumiDicEventListener *listener){
   EventListeners.push_back(listener);  
 }
 
-KasumiWord *KasumiDic::getWordWithID(int id) throw(KasumiOutOfBoundException){
+KasumiWord *KasumiDic::getWordWithID(size_t id)
+  throw(KasumiOutOfBoundException){
+
   if(id >= WordList.size() || id < 0){
     throw new KasumiOutOfBoundException("Out of Bound!");
   }
